(* Ocamllex scanner for YAMML *)

{ open Parser }

rule token = parse
  [' ' '\t' '\r' '\n'] { token lexbuf }         (* Whitespace *)
| "/*"       { block_comment lexbuf }           (* Block Comments *)
| "//"       { sl_comment lexbuf }              (* Single line Comments *)
| '('        { LPAREN }
| ')'        { RPAREN }
| '{'        { LBRACE }
| '}'        { RBRACE }
| '['        { LBRACK }
| ']'        { RBRACK }
| ';'        { SEMI }
| ':'        { COLON }
| ','        { COMMA }
| ".*"       { ELETIMES }
| "./"       { ELEDIVIDE }
| '+'        { PLUS }
| '-'        { MINUS }
| '*'        { TIMES }
| '/'        { DIVIDE }
| '='        { ASSIGN }
| "=="       { EQ }
| "!="       { NEQ }
| '<'        { LT }
| "<="       { LEQ }
| ">"        { GT }
| ">="       { GEQ }
| "&&"       { AND }
| "||"       { OR }
| "!"        { NOT }
| "if"       { IF }
| "else"     { ELSE }
| "for"      { FOR }
| "while"    { WHILE }
| "return"   { RETURN }
| "continue" { CONTINUE }
| "break"    { BREAK }
| "int"      { INT }
| "bool"     { BOOL }
| "void"     { VOID }
| "char"     { CHAR }
| "str"      { STRING }
| "float"    { FLOAT }
| "matrix"   { MATRIX }
| "true"     { TRUE }
| "false"    { FALSE }
| ['0'-'9']+ as lxm { LITERAL(int_of_string lxm) }
| ''' ([^ ''']* as ch) ''' { CHARLIT(String.get (Scanf.unescaped ch) 0) }
| '"' ([^ '"']* as str) '"' { STRINGLIT(Scanf.unescaped str) }
| (((['0'-'9']+'.'['0'-'9']*)|(['0'-'9']*'.'['0'-'9']+))(('e'|'E')('+'|'-')?['0'-'9']+)?|['0'-'9']+(('e'|'E')('+'|'-')?['0'-'9']+)) as lxm { FLOATLIT(float_of_string lxm) }
| ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']* as lxm { ID(lxm) }
| [' ' '\t' '\r' '\n']*'#'[' ' '\t' '\r' '\n']*"import"[' ' '\t' '\r' '\n']*'<' ([^ '<' '>']* as im) '>'{ IMPORT(im) }
| eof { EOF }
| _ as char { raise (Failure("illegal character " ^ Char.escaped char)) }

and block_comment = parse
  "*/" { token lexbuf }
| _    { block_comment lexbuf }

and sl_comment = parse
  ['\n' '\r'] { token lexbuf }
| _    { sl_comment lexbuf }
