(* Top-level of the YAMML compiler: scan & parse the input,
   check the resulting AST, generate LLVM IR, and dump the module *)

module StringMap = Map.Make(String)

type action = Ast | Sast | LLVM_IR | Compile

let _ =
  let action = ref Compile in
  let set_action a () = action := a in
  let speclist = [
    ("-a", Arg.Unit (set_action Ast), "Print the AST");
    ("-s", Arg.Unit (set_action Sast), "Print the SAST");
    ("-l", Arg.Unit (set_action LLVM_IR), "Print the generated LLVM IR");
    ("-c", Arg.Unit (set_action Compile),
      "Check and print the generated LLVM IR (default)");
  ] in  
  let usage_msg = "usage: ./yamml.native [-a|-l|-c] [file.yamml]" in
  let channel = ref stdin in
  Arg.parse speclist (fun filename -> channel := open_in filename) usage_msg;

  let lexbuf channel = Lexing.from_channel !channel in
  let rec ast channel = 
    let (imports, g, f) = Parser.program Scanner.token (lexbuf channel) in
    let import (gl, fu) im =
      let ichannel = ref (open_in (match im with Ast.Import f -> f)) in
      let iast = ast ichannel in
      (fst iast @ gl, snd iast @ fu)
    in
    List.fold_left import (g, f) imports
  in
  match !action with
    Ast -> print_string (Ast.string_of_program (ast channel))
  | _ -> let sast = Semant.check (ast channel) in
    match !action with
      Ast     -> () 
    | Sast    -> print_string (Sast.string_of_sprogram sast)
    | LLVM_IR -> print_string (Llvm.string_of_llmodule (Codegen.translate sast))
    | Compile -> let m = Codegen.translate sast in
  Llvm_analysis.assert_valid_module m;
  print_string (Llvm.string_of_llmodule m)

